
#include <Arduino.h>
#include <DHT.h>
#include <DHT_U.h>

#define APIKEY "OTI5YUPI0PALJR3F"

//view page: https://thingspeak.com/channels/825092 
/**
 * Maduino Zero SIM800C AT Commands Demo
 * Author:Charlin
 * Date:2019/08/06
 * Version:v1.1
 * Website:www.makerfabs.com
 * Email:Charlin@makerfabs.com
 * Function: 
 * Get humidity and temperature from DHT11 sensor. Post data to website via GSM/GPRS network
 * So upload this firmware to board, you can view the data on the web page  https://thingspeak.com/channels/825092
 */


#define DEBUG true //true: debug on; false:debug off
int POWER_KEY=9; //D9

bool SIM800C_ON = false;//

//*******for senser **********
float humidity;//humidity
float temperature;//temperature

#define DHTPIN 3    //D3
#define DHTTYPE DHT11 
DHT dht(DHTPIN, DHTTYPE);

void setup()
{
  pinMode(POWER_KEY, OUTPUT);

  //output 1 second pulse to turn on the SIM800C
  digitalWrite(POWER_KEY, HIGH);
  delay(1000);
  digitalWrite(POWER_KEY, LOW);
  
  SerialUSB.begin(115200);
  delay(100);

  //Waiting for Arduino Serial Monitor port to connect, if you use other serial tool, you can commenting this.
//  while (!SerialUSB) {
//    delay(100);
//  }
  
  if(DEBUG){
    SerialUSB.println("program starts to run!");
  }
  Serial1.begin(115200);

  //********check SIM800C whether it open************
  sendData("AT", 1000, DEBUG);
  delay(1000);
  //******************************

  //********open*********
  if (!SIM800C_ON){
    digitalWrite(POWER_KEY, LOW);    
    delay(1000); 
    digitalWrite(POWER_KEY, HIGH);
    if(DEBUG){
      SerialUSB.println("Now turning on SIM800C");
    }
    delay(16000);
  }
  //*********************

  sendData("AT+CGREG=0", 1000, DEBUG);
  sendData("AT+CREG=0", 1000, DEBUG);
  //**************Open internet connection*************************  
  sendData("AT+CGATT=1",1000,DEBUG);//Attach to GPRS service
  delay(2000); 
  sendData("AT+SAPBR=3,1,\"CONTYPE\",\"GPRS\"", 1000, DEBUG);
  sendData("AT+SAPBR=3,1,\"APN\",\"CMNET\"", 1000, DEBUG);
  sendData("AT+SAPBR=1,1", 1000, DEBUG);
  
  //***************************************************************
   sendData("AT+HTTPINIT", 1000, DEBUG);//initalize HTTP Service

  dht.begin();  //DHT init
   
}

//send data to website every 60 seconds
void loop()
{
  /**
  while (Serial1.available() > 0) {
    SerialUSB.write(Serial1.read());
    yield();
    }
  while (SerialUSB.available() > 0) {
    Serial1.write(SerialUSB.read());
    yield();
    }
  **/

  temperature = dht.readTemperature(); //get temperature
  humidity = dht.readHumidity();  //get humidity
  
                       
  SerialUSB.print("temperature is :");
  SerialUSB.println(temperature);

  SerialUSB.print("humidity is :");
  SerialUSB.println(humidity);
  
  String command = "AT+HTTPPARA=\"URL\",\"http://api.thingspeak.com/update.json?api_key=" + (String)APIKEY + "&field1=" + (String)temperature +"&field2=" + (String)humidity + "\"";
  //String command = "AT+HTTPPARA=\"URL\",\"http://api.thingspeak.com/update.json?api_key=" + (String)APIKEY + "&field1=25.5&field2=67.8\"";
  
  //SerialUSB.println(command);
  
  //Set HTTP Parameters Value 
  sendData(command, 1000, DEBUG);
  delay(1000); 
  sendData("AT+HTTPACTION=1", 5000, DEBUG);//0 get, 1 post, 2 head
  delay(60000);
}


String sendData(String command, const int timeout, boolean debug)
{
    String response = "";    
    Serial1.println(command);

    String commandpre = getcommand_pref(command);
    //SerialUSB.println(commandpre);

    long int time = millis();   
    while( (time+timeout) > millis())
    {
       if(commandpre == "AT"){       
        if(Serial1.find("OK")){
          SIM800C_ON = true;
          SerialUSB.println("SIM800C is ON");
        }
      }

      while(Serial1.available())
      {       
        String c = Serial1.readString(); 
        response+=c;
      } 
   }   

   if(debug){
      SerialUSB.println(response);
   }
   return response;
}


String getcommand_pref(String command){

    String command_pref = "";
    char *cstr = new char[command.length() + 1];
    strcpy(cstr, command.c_str());
    char * token = strtok(cstr,"=");
    int i = 0;
    while(token != NULL){
      //SerialUSB.print(token);
      //SerialUSB.println("  line" + (String)i);

      switch(i) {
        case 0:
          command_pref = token;
        break;
        
        default:
        break;
      }

      token = strtok(NULL,",");
      i = i+1;
    }

    if(command_pref == "")
    command_pref = command;
    
    return command_pref;
}
